import numpy as np
import mergechannels as mc


def test_apply_color_map():
    '''
    Test that the color map is applied correctly
    '''
    x = np.ones((1, 1), dtype=np.uint8)
    rgb = mc.apply_color_map(x, 'betterBlue')
    assert rgb.shape == (1,1,3)
    assert rgb.dtype == np.uint8
    assert np.allclose(
        rgb,
        np.array(
            [[[0, 1, 2]]]
        )
    )
    x = np.ones((1, 1), dtype=np.uint8) * 255
    rgb = mc.apply_color_map(x, 'betterBlue')
    assert np.allclose(
        rgb,
        np.array(
            [[[0, 188, 254]]]
        )
    )

def test_apply_colors_and_merge_low_sum():
    '''
    Test that the colors are merged correctly with sum blending and low values
    '''
    x = np.ones((1, 1), dtype=np.uint8)
    y = np.ones((1, 1), dtype=np.uint8)
    rgb_sum = mc.apply_colors_and_merge_nc([x, y], ['betterBlue', 'betterOrange'], 'sum')
    # blue = [0, 1, 2,]
    # orange = [1, 1, 0]
    assert np.allclose(
        rgb_sum,
        np.array(
            [[[1, 2, 2]]]
        )
    )

def test_apply_colors_and_merge_high_sum():
    '''
    Test that the colors are merged correctly with sum blending and high values
    '''
    x = np.ones((1, 1), dtype=np.uint8) * 255
    y = np.ones((1, 1), dtype=np.uint8) * 255
    rgb_sum = mc.apply_colors_and_merge_nc([x, y], ['betterBlue', 'betterOrange'], 'sum')
    # blue = [0, 188, 254]
    # orange = [255, 149, 0]
    assert np.allclose(
        rgb_sum,
        np.array(
            [[[255, 255, 254]]]
        )
    )

def test_apply_colors_and_merge_low_max():
    '''
    Test that the colors are merged correctly with max blending and low values
    '''
    x = np.ones((1, 1), dtype=np.uint8)
    y = np.ones((1, 1), dtype=np.uint8)
    rgb_max = mc.apply_colors_and_merge_nc([x, y], ['betterBlue', 'betterOrange'], 'max')
    # blue = [0, 1, 2,]
    # orange = [1, 1, 0]
    assert np.allclose(
        rgb_max,
        np.array(
            [[[1, 1, 2]]]
        )
    )

def test_apply_colors_and_merge_high_max():
    '''
    Test that the colors are merged correctly with max blending and high values
    '''
    x = np.ones((1, 1), dtype=np.uint8) * 255
    y = np.ones((1, 1), dtype=np.uint8) * 255
    rgb_max = mc.apply_colors_and_merge_nc([x, y], ['betterBlue', 'betterOrange'], 'max')
    # blue = [0, 188, 254]
    # orange = [255, 149, 0]
    assert np.allclose(
        rgb_max,
        np.array(
            [[[255, 188, 254]]]
        )
    )

def test_apply_colors_and_merge_low_min():
    '''
    Test that the colors are merged correctly with min blending and low values
    '''
    x = np.ones((1, 1), dtype=np.uint8)
    y = np.ones((1, 1), dtype=np.uint8)
    rgb_min = mc.apply_colors_and_merge_nc([x, y], ['betterBlue', 'betterOrange'], 'min')
    # blue = [0, 1, 2,]
    # orange = [1, 1, 0]
    assert np.allclose(
        rgb_min,
        np.array(
            [[[0, 1, 0]]]
        )
    )

def test_apply_colors_and_merge_high_min():
    '''
    Test that the colors are merged correctly with min blending and high values
    '''
    x = np.ones((1, 1), dtype=np.uint8) * 255
    y = np.ones((1, 1), dtype=np.uint8) * 255
    rgb_min = mc.apply_colors_and_merge_nc([x, y], ['betterBlue', 'betterOrange'], 'min')
    # blue = [0, 188, 254]
    # orange = [255, 149, 0]
    assert np.allclose(
        rgb_min,
        np.array(
            [[[0, 149, 0]]]
        )
    )

def test_apply_colors_and_merge_low_mean():
    '''
    Test that the colors are merged correctly with mean blending and low values
    '''
    x = np.ones((1, 1), dtype=np.uint8)
    y = np.ones((1, 1), dtype=np.uint8)
    rgb_mean = mc.apply_colors_and_merge_nc([x, y], ['betterBlue', 'betterOrange'], 'mean')
    # blue = [0, 1, 2,]
    # orange = [1, 1, 0]
    assert np.allclose(
        # NOTE: I haven't decided if I'm going to re-normalize after blending so this test may fail
        # in the future
        rgb_mean,
        np.array(
            [[[0, 1, 1]]]
        )
    )

def test_apply_colors_and_merge_high_mean():
    '''
    Test that the colors are merged correctly with mean blending and high values
    '''
    x = np.ones((1, 1), dtype=np.uint8) * 255
    y = np.ones((1, 1), dtype=np.uint8) * 255
    rgb_mean = mc.apply_colors_and_merge_nc([x, y], ['betterBlue', 'betterOrange'], 'mean')
    # blue = [0, 188, 254]
    # orange = [255, 149, 0]
    assert np.allclose(
        # NOTE: I haven't decided if I'm going to re-normalize after blending so this test may fail
        # in the future
        rgb_mean,
        np.array(
            [[[127, 168, 127]]]
        )
    )



from typing import Sequence

import numpy as np

from mergechannels import (
	apply_color_map,
	apply_colors_and_merge_nc,
)
from ._luts import COLORMAPS
from ._blending import BLENDING_OPTIONS

def merge(
	arrs: Sequence[np.ndarray],
	colors: Sequence[COLORMAPS],
	blending: BLENDING_OPTIONS = 'max',
) -> np.ndarray:
	'''
	apply cmaps to arrays and blend the colors
	'''
	# region validation
	n_arrs = len(arrs)
	n_colors = len(colors)
	if not n_arrs == n_colors and n_arrs > 0:
		raise ValueError(
			'Expected an equal number of arrays to colorize and colormap names to apply. '
			f'Got {n_arrs} arrays and {n_colors} colors'
		)
	arr_shapes = [arr.shape for arr in arrs]
	if not len(set(arr_shapes)) == 1:
		raise ValueError(
			f'Expected every array to have the same shape, got {arr_shapes}'
		)
	if not len(arr_shapes[0]) == 2:
		raise ValueError(
			f'Expected every array to be 2D, got {arr_shapes[0]}'
		)
	arr_dtypes = [arr.dtype for arr in arrs]
	if not len(set(arr_dtypes)) == 1:
		raise ValueError(
			f'Expected every array to have the same dtype, got {arr_dtypes}'
		)
	# endregion

	if n_arrs == 1:
		return apply_color_map(arr=arrs[0], cmap_name=colors[0])
	else:
		return apply_colors_and_merge_nc(arrs, colors, blending)



from typing import Literal

COLORMAPS = Literal[
	'Rainbow RGB',
	'3color-RMB',
	'Red Hot',
	'Yellow',
	'betterRed',
	'blue_orange_icb',
	'mpl-inferno',
	'I Blue',
	'BOP purple',
	'betterOrange',
	'Blue',
	'6_shades',
	'OIMB3',
	'Red/Green',
	'Turbo',
	'mpl-viridis',
	'smart',
	'Thermal',
	'OIMB2',
	'glow',
	'OPF orange',
	'physics',
	'OIMB1',
	'betterCyan',
	'mpl-magma',
	'edges',
	'OPF purple',
	'I Yellow',
	'16_colors',
	'sepia',
	'ICA2',
	'I Forest',
	'ICA3',
	'BOP orange',
	'I Red',
	'3color-CGY',
	'I Green',
	'Fire',
	'Magenta Hot',
	'Orange Hot',
	'unionjack',
	'I Cyan',
	'Cyan Hot',
	'I Magenta',
	'Grays',
	'Cyan',
	'phase',
	'gem',
	'OPF fresh',
	'betterGreen',
	'3color-BMR',
	'betterBlue',
	'Magenta',
	'Red',
	'I Bordeaux',
	'glasbey',
	'glasbey_on_dark',
	'mpl-plasma',
	'3color-YGC',
	'Green',
	'ICA',
	'Yellow Hot',
	'thallium',
	'Spectrum',
	'royal',
	'glasbey_inverted',
	'betterYellow',
	'HiLo',
	'cool',
	'Green Fire Blue',
	'BOP blue',
	'thal',
	'Ice',
	'brgbcmyw',
	'I Purple',
	'3-3-2 RGB',
	'5_ramps',
]



from typing import Sequence, Literal

import numpy as np

from ._luts import COLORMAPS
from ._blending import BLENDING_OPTIONS

def merge(
	arrs: Sequence[np.ndarray],
	colors: Sequence[COLORMAPS] = (),
	blending: Literal[BLENDING_OPTIONS] = 'max',
    saturation_limits: tuple[float, float] = (0.2, 99.8),
) -> np.ndarray:
    ...

def apply_color_map(
    arr: np.ndarray,
    cmap_name: COLORMAPS,
    saturation_limits: tuple[float, float] = (0.2, 99.8),
) -> np.ndarray: ...

def apply_colors_and_merge_nc(
    arrs: Sequence[np.ndarray],
    colors: Sequence[COLORMAPS] = (),
    blending: Literal[BLENDING_OPTIONS] = 'max',
    saturation_limits: Sequence[tuple[float, float]] = (),
) -> np.ndarray: ...



# /// script
# requires-python = ">=3.11"
# dependencies = [
#     "numpy",
# ]
# ///

'''
This script converts the luts in the `assets/converted` directory into
1) a lazy static hashmap that with the rgb values for each lut
2) a type hint file for the lut names so that the names are autocompleted in IDEs

This script can be invoked with `uv run scripts/populate_luts.py`
'''

from pathlib import Path
import numpy as np


def main():
    '''
    Build the lazy static hashmap and the type hint file for the lut names
    '''
    curr_file = Path(__file__).absolute()
    luts_dir = curr_file.parent.parent / 'assets' / 'converted'
    lut_files = list(luts_dir.glob('*.txt'))
    lut_data: dict[str, np.ndarray] = {}
    for lut_file in lut_files:
        with open(lut_file, 'r') as f:
            lut_vals = []
            for line in f.readlines():
                line_vals = np.array(line.strip().split('\t'), dtype='uint8')
                assert len(line_vals) == 3
                lut_vals.append(line_vals)
            lut = np.array(lut_vals, dtype='uint8')
            assert lut.shape == (256, 3)
            lut_data[lut_file.stem.replace('%', '/')] = lut

    cmaps_file = curr_file.parent.parent / 'src' / 'cmaps.rs'
    assert cmaps_file.exists()

    cmaps_file_lines = ''
    cmaps_file_lines += 'use lazy_static::lazy_static;\n'
    cmaps_file_lines += 'use std::collections::HashMap;\n\n'
    cmaps_file_lines += 'type Colormap = [[u8; 3]; 256];\n\n'
    cmaps_file_lines += 'lazy_static! {\n'
    cmaps_file_lines += "\tpub static ref CMAPS: HashMap<&'static str, Colormap> = {\n"
    cmaps_file_lines += '\t\tlet mut m = HashMap::new();\n'

    for lut_name, lut in lut_data.items():
        cmaps_file_lines += f'\t\tm.insert("{lut_name}", [\n'
        for i in range(256):
            cmaps_file_lines += f'\t\t\t[{lut[i, 0]}, {lut[i, 1]}, {lut[i, 2]}],\n'
        cmaps_file_lines += '\t\t]);\n\n'
    cmaps_file_lines += '\t\tm'
    cmaps_file_lines += '\t};'
    cmaps_file_lines += '}'

    with open(cmaps_file, 'w') as f:
        f.write(cmaps_file_lines)

    lut_names_type_hint_file = curr_file.parent.parent / 'python' / 'mergechannels' / '_luts.py'
    lut_names_lines = 'from typing import Literal\n\n'
    lut_names_lines += 'COLORMAPS = Literal[\n'
    for lut_name in lut_data:
        lut_names_lines += f"\t'{lut_name}',\n"
    lut_names_lines += ']\n'

    with open(lut_names_type_hint_file, 'w') as f:
        f.write(lut_names_lines)


if __name__ == '__main__':
    main()



# Import Rust functions
from .mergechannels import (  # type: ignore
	apply_color_map,
	apply_colors_and_merge_nc,
)

from ._internal import merge

__all__ = [
	'apply_color_map',
	'apply_colors_and_merge_nc',
	'merge',
]
